/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.petals.clientserverapi.topology.TopologyService.DomainPropertyName;
import org.ow2.petals.clientserverapi.topology.TopologyService.DomainPropertyValues;

/**
 * This class holds the global topology information.
 * <p>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 * </p>
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @since Petals 2.0
 */
public class DomainConfiguration implements Serializable {

    private static final long serialVersionUID = -4951508542017620408L;

    /**
     * domain mode enumeration.
     */
    public static enum DomainMode {

        /**
         * Static distributed mode
         */
        STATIC, /**
                 * Dynamic distributed mode where nodes can be attached or detached at runtime
                 */
        DYNAMIC
    };

    /**
     * Domain name.
     */
    private String name;

    /**
     * Domain description
     */
    private String description;

    /**
     * The JNDI configuration.
     */
    private JndiConfiguration jndiConfiguration;

    /**
     * Domain mode.
     */
    private DomainMode mode;

    /**
     * Implementation of the shared area shared by nodes of the domain
     */
    private String sharedAreaImplementation;

    /**
     * Extra configuration parameters
     */
    private List<Object> extraParameters;

    public String getName() {
        return this.name;
    }

    public String getDescription() {
        return this.description;
    }

    public JndiConfiguration getJndiConfiguration() {
        return this.jndiConfiguration;
    }

    public DomainMode getMode() {
        return this.mode;
    }

    public String getSharedAreaImplementation() {
        return this.sharedAreaImplementation;
    }

    public List<Object> getExtraParameters() {
        return this.extraParameters;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setJndiConfiguration(JndiConfiguration jndiConfiguration) {
        this.jndiConfiguration = jndiConfiguration;
    }
    
    public void setMode(DomainMode mode) {
        this.mode = mode;
    }

    public void setSharedAreaImplementation(final String sharedAreaImplementation) {
        this.sharedAreaImplementation = sharedAreaImplementation;
    }

    public void setExtraParameters(final List<Object> extraParameters) {
        this.extraParameters = extraParameters;
    }

    /**
     * Convert the domain configuration into a key/value map form.
     * 
     * @return The domain configuration as key/value map form.
     */
    public Map<String, String> toMap() {

        final Map<String, String> domainMap = new HashMap<String, String>();
        domainMap.put(DomainPropertyName.CONF_DOMAIN_NAME, this.getName());
        if (this.getDescription() != null) {
            domainMap.put(DomainPropertyName.CONF_DOMAIN_DESCRIPTION, this.getDescription());
        } else {
            domainMap.put(DomainPropertyName.CONF_DOMAIN_DESCRIPTION, "");
        }

        if (DomainConfiguration.DomainMode.DYNAMIC.equals(this.getMode())) {
            domainMap.put(DomainPropertyName.CONF_DOMAIN_MODE, DomainPropertyValues.MODE_DYNAMIC);
        } else {
            domainMap.put(DomainPropertyName.CONF_DOMAIN_MODE, DomainPropertyValues.MODE_STATIC);
        }

        if (this.getSharedAreaImplementation() != null) {
            domainMap.put(DomainPropertyName.CONF_SHARED_AREA_IMPLEMENTATION, this.getSharedAreaImplementation());
        } else {
            domainMap.put(DomainPropertyName.CONF_SHARED_AREA_IMPLEMENTATION, "");
        }

        final JndiConfiguration jndiConfiguration = this.getJndiConfiguration();
        if (jndiConfiguration != null) {
            domainMap.putAll(jndiConfiguration.toMap());
        } else {
            domainMap.put(DomainPropertyName.CONF_JNDI_FACTORY, "");
            domainMap.put(DomainPropertyName.CONF_JNDI_PROVIDER_URL, "");
            domainMap.put(DomainPropertyName.CONF_JNDI_SECURITY_PRINCIPAL, "");
            domainMap.put(DomainPropertyName.CONF_JNDI_SECURITY_CREDENTIALS, "");
            domainMap.put(DomainPropertyName.CONF_JNDI_POOL_SIZE, "");
            domainMap.put(DomainPropertyName.CONF_JNDI_BATCH_SIZE, "");

        }
        return domainMap;
    }

    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Domain ").append(this.name);
        if (this.description != null) {
            toStringBuilder.append("\n\t\t\tDescription :\n\t\t\t  ").append(this.description);
        }
        toStringBuilder.append("\n\t\t\t  Configuration :");
        toStringBuilder.append("\n\t\t\t  - JNDI configuration :" + this.jndiConfiguration);
        if (this.sharedAreaImplementation != null && !this.sharedAreaImplementation.isEmpty()) {
            toStringBuilder.append("\n\t\t\t\t- Shared area implementation:" + this.sharedAreaImplementation);
        } else {
            toStringBuilder
                    .append("\n\t\t\t\t- Shared area implementation: not set, will be look up in the classloader");
        }
        if (this.extraParameters != null && !this.extraParameters.isEmpty()) {
            toStringBuilder.append("\n\t\t\t\t- Shared area implementation extra parameters:" + this.extraParameters);
        } else {
            toStringBuilder.append("\n\t\t\t\t- Shared area implementation: no extra parameter");
        }
        return toStringBuilder.toString();
    }

}
